import os
import pandas as pd
from sklearn.preprocessing import StandardScaler
import logging
from sklearn.model_selection import train_test_split
from feature_engineering import add_features
import sys

# 프로젝트 루트 및 스크립트 경로 설정
project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
scripts_dir = os.path.join(project_root, 'scripts')
config_dir = os.path.join(project_root, 'config')
utils_dir = os.path.join(scripts_dir, 'utils')

sys.path.extend([
    scripts_dir,
    os.path.join(scripts_dir, 'data_collection'),
    os.path.join(scripts_dir, 'data_processing'),
    os.path.join(scripts_dir, 'model'),
    os.path.join(scripts_dir, 'trading'),
    utils_dir
])

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'scripts', 'model')))
from model_training import update_lstm_model, train_lstm_model

logging.basicConfig(level=logging.INFO)

def normalize_data(df):
    """Normalize the data using StandardScaler."""
    scaler = StandardScaler()
    normalized_df = pd.DataFrame(scaler.fit_transform(df), columns=df.columns)
    return normalized_df, scaler

def load_csv_files_from_directory(directory_path):
    dataframes = []
    for file_name in os.listdir(directory_path):
        if file_name.endswith('.csv'):
            file_path = os.path.join(directory_path, file_name)
            try:
                df = pd.read_csv(file_path, encoding='utf-8')
                logging.info(f"Loaded {file_name} with columns: {df.columns}")
                df.columns = df.columns.str.lower()  # Make all column names lowercase
                if not df.empty:
                    logging.info(f"First few rows of {file_name}:\n{df.head()}")
                    # Check for required columns based on the type of data
                    if 'news_data' in directory_path:
                        required_columns = ['title', 'link', 'date', 'stock_name', 'stock_code']
                    elif 'financial_data' in directory_path:
                        required_columns = ['date', 'account_nm', 'thstrm_amount', 'frmtrm_amount']
                    elif 'economic_data' in directory_path and 'gdp_data' in file_name:
                        required_columns = ['date', 'item code', 'item name', 'gdp']
                    elif 'economic_data' in directory_path and 'inflation_rate_data' in file_name:
                        required_columns = ['date', 'item code', 'item name', 'inflation rate']
                    else:
                        required_columns = ['open', 'high', 'low', 'close', 'volume', 'date']
                    
                    if all(col in df.columns for col in required_columns):
                        dataframes.append(df)
                    else:
                        missing_cols = set(required_columns) - set(df.columns)
                        logging.error(f"Missing required columns {missing_cols} in {file_path}")
                else:
                    logging.warning(f"Empty file: {file_path}")
            except pd.errors.EmptyDataError:
                logging.warning(f"Empty file: {file_path}")
            except Exception as e:
                logging.error(f"Error reading {file_path}: {e}")
    return pd.concat(dataframes, ignore_index=True) if dataframes else pd.DataFrame()

def preprocess_df(df, date_column='date', date_format=None):
    if date_column not in df.columns:
        raise ValueError(f"Date column '{date_column}' not found in the dataset. Available columns: {df.columns}")
    
    logging.info(f"Before date conversion, first few rows:\n{df.head()}")

    # 날짜 형식이 지정되면, 해당 형식으로 변환
    if date_format:
        df[date_column] = pd.to_datetime(df[date_column], format=date_format, errors='coerce')
    else:
        df[date_column] = pd.to_datetime(df[date_column], errors='coerce')

    logging.info(f"After date conversion, first few rows:\n{df.head()}")

    # 날짜가 변환되지 않은 (NaT로 남은) 행들을 제거
    df.dropna(subset=[date_column], inplace=True)

    logging.info(f"After dropping rows with invalid dates, shape: {df.shape}")

    df.fillna(0, inplace=True)
    return df

def preprocess_data(stock_data, financial_data, news_data, economic_data):
    if stock_data is None or stock_data.empty:
        logging.error("Stock data is empty or None. Cannot proceed with preprocessing.")
        return None
    
    logging.info(f"Initial stock data shape: {stock_data.shape}")
    
    # 주식 데이터는 '%Y-%m-%d' 형식으로 변환
    stock_data = preprocess_df(stock_data, date_column='date', date_format='%Y-%m-%d')
    
    logging.info(f"Stock data after preprocessing shape: {stock_data.shape}")
    
    # 재무 데이터는 '%Y%m%d' 형식으로 변환 (이 부분은 데이터셋에 맞춰 변경)
    if financial_data is not None and not financial_data.empty:
        financial_data = preprocess_df(financial_data, date_column='date', date_format='%Y%m%d')
        logging.info(f"Financial data shape: {financial_data.shape}")
    else:
        financial_data = pd.DataFrame()
    
    # 뉴스 데이터는 '%Y%m%d' 형식으로 변환
    if news_data is not None and not news_data.empty:
        news_data = preprocess_df(news_data, date_column='date', date_format='%Y%m%d')
        logging.info(f"News data shape: {news_data.shape}")
    else:
        news_data = pd.DataFrame()
    
    # 경제 데이터는 '%Y%m%d' 형식으로 변환 (필요에 따라 맞춤 설정)
    if economic_data is not None and not economic_data.empty:
        economic_data = preprocess_df(economic_data, date_column='date', date_format='%Y%m%d')
        logging.info(f"Economic data shape: {economic_data.shape}")
    else:
        economic_data = pd.DataFrame()

    stock_data['target'] = stock_data['close'].shift(-1)
    stock_data.dropna(subset=['target'], inplace=True)

    if stock_data.empty:
        logging.error("No valid stock data after preprocessing.")
        return None

    # 데이터 병합 단계
    if not financial_data.empty:
        stock_data = pd.merge_asof(stock_data.sort_values('date'), financial_data.sort_values('date'), on='date', direction='nearest')
        logging.info(f"Stock data shape after merging with financial data: {stock_data.shape}")
    
    if not economic_data.empty:
        stock_data = pd.merge_asof(stock_data.sort_values('date'), economic_data.sort_values('date'), on='date', direction='nearest')
        logging.info(f"Stock data shape after merging with economic data: {stock_data.shape}")
    
    if not news_data.empty:
        stock_data = pd.merge_asof(stock_data.sort_values('date'), news_data.sort_values('date'), on='date', direction='nearest')
        logging.info(f"Stock data shape after merging with news data: {stock_data.shape}")

    if stock_data.empty:
        logging.error("No valid combined data after merging.")
        return None

    # 피처 엔지니어링
    stock_data = add_features(stock_data)

    for col in stock_data.columns:
        if stock_data[col].dtype == 'object':
            stock_data[col] = pd.to_numeric(stock_data[col].str.replace(',', ''), errors='coerce')

    stock_data.ffill(inplace=True)
    stock_data.bfill(inplace=True)

    stock_data.fillna(0, inplace=True)

    X = stock_data.drop(['target', 'date'], axis=1)
    y = stock_data['target']

    if X.empty or y.empty:
        logging.error("Insufficient data after preprocessing for training the model.")
        return None

    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)
    scaler = StandardScaler()
    X_train = scaler.fit_transform(X_train)
    X_test = scaler.transform(X_test)

    return (X_train, X_test, y_train, y_test), scaler, stock_data



def main(base_dir):
    stock_data_dir = os.path.join(base_dir, 'stock_data')
    financial_data_dir = os.path.join(base_dir, 'financial_data')
    news_data_dir = os.path.join(base_dir, 'news_data')
    economic_data_dir = os.path.join(base_dir, 'economic_data')

    # 주식 데이터 로드 후 초기 상태 확인
    stock_data = load_csv_files_from_directory(stock_data_dir)
    if stock_data.empty:
        logging.error("Stock data is missing or empty after loading.")
        return
    logging.info(f"Stock data initial state: {stock_data.head()}")

    financial_data = load_csv_files_from_directory(financial_data_dir)
    news_data = load_csv_files_from_directory(news_data_dir)
    economic_data = load_csv_files_from_directory(economic_data_dir)

    if financial_data.empty:
        logging.warning("Financial data is missing or empty.")
    if news_data.empty:
        logging.warning("News data is missing or empty.")
    if economic_data.empty:
        logging.warning("Economic data is missing or empty.")
    
    # Preprocess original data and save
    result_original = preprocess_data(stock_data, financial_data, news_data, economic_data)
    
    if result_original is None:
        logging.error("Data preprocessing failed due to insufficient data.")
        return

    (_, _, _, _), _, combined_data_original = result_original

    if combined_data_original is None or combined_data_original.empty:
        logging.error("No valid combined data after preprocessing.")
        return

    output_path_original = os.path.join(base_dir, 'preprocessed_data.csv')
    combined_data_original.to_csv(output_path_original, index=False)
    logging.info(f"Combined data saved to {output_path_original}")

    # Preprocess only stock data and save with "_stock" suffix
    result_stock = preprocess_data(stock_data, pd.DataFrame(), pd.DataFrame(), pd.DataFrame())
    
    if result_stock is None:
        logging.error("Stock data preprocessing failed due to insufficient data.")
        return

    (_, _, _, _), _, combined_data_stock = result_stock

    if combined_data_stock is None or combined_data_stock.empty:
        logging.error("No valid stock data after preprocessing.")
        return

    output_path_stock = os.path.join(base_dir, 'preprocessed_data_stock.csv')
    combined_data_stock.to_csv(output_path_stock, index=False)
    logging.info(f"Stock data saved to {output_path_stock}")

if __name__ == "__main__":
    base_dir = 'C:\\Users\\user\\projects\\st\\data'  # 사용자의 데이터 디렉토리 경로
    main(base_dir)

