import pandas as pd
import logging

def add_technical_indicators(data):
    required_columns = ['close', 'high', 'low', 'volume']
    
    for column in required_columns:
        if column not in data.columns:
            logging.error(f"Missing column: {column} in data")
            return data
    
    # Simple Moving Averages
    data['MA_20'] = data['close'].rolling(window=20).mean()
    data['MA_50'] = data['close'].rolling(window=50).mean()

    # Bollinger Bands
    data['BB_upper'] = data['MA_20'] + 2 * data['close'].rolling(window=20).std()
    data['BB_lower'] = data['MA_20'] - 2 * data['close'].rolling(window=20).std()

    # Moving Average Convergence Divergence (MACD)
    data['MACD'] = data['close'].ewm(span=12, adjust=False).mean() - data['close'].ewm(span=26, adjust=False).mean()
    data['MACD_signal'] = data['MACD'].ewm(span=9, adjust=False).mean()

    # Relative Strength Index (RSI)
    delta = data['close'].diff()
    gain = (delta.where(delta > 0, 0)).rolling(window=14).mean()
    loss = (-delta.where(delta < 0, 0)).rolling(window=14).mean()
    rs = gain / loss
    data['RSI'] = 100 - (100 / (1 + rs))

    # Stochastic Oscillator
    low_14 = data['low'].rolling(window=14).min()
    high_14 = data['high'].rolling(window=14).max()
    data['%K'] = (data['close'] - low_14) * 100 / (high_14 - low_14)
    data['%D'] = data['%K'].rolling(window=3).mean()

    # Momentum
    data['Momentum'] = data['close'] - data['close'].shift(10)

    # Average True Range (ATR)
    high_low = data['high'] - data['low']
    high_close = (data['high'] - data['close'].shift()).abs()
    low_close = (data['low'] - data['close'].shift()).abs()
    ranges = pd.concat([high_low, high_close, low_close], axis=1)
    true_range = ranges.max(axis=1)
    data['ATR'] = true_range.rolling(window=14).mean()

    # Volume Weighted Average Price (VWAP)
    data['VWAP'] = (data['close'] * data['volume']).cumsum() / data['volume'].cumsum()

    # Drop NA values after adding indicators
    data.dropna(inplace=True)

    return data

def add_features(data):
    data = add_technical_indicators(data)
    return data

if __name__ == '__main__':
    stock_data = pd.read_csv('/path/to/your/stock_data.csv')  # 예시 경로입니다. 실제 경로로 변경하세요.
    stock_data = add_features(stock_data)
    print(stock_data.head())
